package com.huoran.iasf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.iasf.common.exception.BusinessException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.Category;
import com.huoran.iasf.entity.ProductDetails;
import com.huoran.iasf.service.SysCategoryService;
import com.huoran.iasf.service.SysProductDetailsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 产品与课程分类表控制类
 * 提供分类管理相关接口
 *
 * @author Rong
 * @date 2024-06-12
 */
@RestController
@RequestMapping("/categoryService/Category")
@Api(value = "产品与课程分类管理接口", tags = "产品与课程分类管理")
public class CategoryController {

    @Autowired
    private SysCategoryService service;

    @Autowired
    private SysProductDetailsService productDetailsService;


    /**
     * 保存或更新分类信息
     *
     * @param category 分类对象
     * @return 操作结果
     */
    @PostMapping("/saveOrUpdate")
    @ApiOperation("保存或更新分类")
    @Transactional(rollbackFor = Exception.class)
    public R saveOrUpdate(@RequestBody @Validated Category category) {
        try {
            // 增加重复检查逻辑
            boolean isDuplicate = false;
            if (category.getCategoryId() != null) {
                // 更新逻辑，排除当前记录自身
                isDuplicate = service.existsOtherByName(category.getName(), category.getCategoryId(),
                        category.getSiteId(), category.getType());
            } else {
                // 新增逻辑，直接检查名称是否存在
                isDuplicate = service.existsByName(category.getName(), category.getSiteId(), category.getType());
            }

            if (isDuplicate) {
                return R.fail("当前名称已存在，请检查后重新提交");
            }

            boolean operationResult = service.saveOrUpdate(category);

            // 在执行操作并获取新ID之后
            long newId = category.getCategoryId();// 这里是从数据库操作中获取到的新插入记录的ID
            return operationResult ? R.success(newId, "操作成功，新增记录ID为: " + newId) : R.fail("操作失败");
        } catch (Exception e) {
            throw new BusinessException(BaseResponseCode.SYSTEM_BUSY);
        }
    }


    /**
     * 获取分类列表，支持类型筛选
     *
     * @param type 类型标识，如"course"或"product"
     * @return 分类列表
     */
    @PostMapping("/list")
    @ApiOperation("根据类型获取分类列表")
    public R categoryList(@ApiParam(value = "类型标识") @RequestParam(required = false) Integer type, @ApiParam(value =
            "站点id") @RequestParam(required = false) Integer siteId) {
        QueryWrapper<Category> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("type", type);
        queryWrapper.eq("site_id", siteId);
        List<Category> categories = service.list(queryWrapper);
        return R.success(categories);
    }


    @PostMapping("/hasBeenCitedInTheArticle")
    @ApiOperation("已被文章引用的(在前台展示)")
    public R hasBeenCitedInTheArticle(@ApiParam(value = "类型标识") @RequestParam(required = false) Integer type,
                                      @ApiParam(value = "站点id") @RequestParam(required = false) Integer siteId) {

        Set<Category> categories = new HashSet<>();

        if (type == 0) {
            // 类型
            List<Category> categoriesList = service.lookUpTheTypeOfReferenceBySite(siteId, type);

            // 将List转换为Set
            categories.addAll(categoriesList);
            return R.success(categories);

        } else {
            // 分类、标签
            String tagIds = service.queryLabelsReferencedWithinSite(siteId, type);
            if (!StringUtils.isEmpty(tagIds)) {
                // 逗号循环分隔设置到集合中
                String[] tagArray = tagIds.split(",");
                for (String tagId : tagArray) {
                    Category category = service.getById(tagId);
                    if (!ObjectUtils.isEmpty(category) && category.getType() == 1) {
                        categories.add(category);
                    }
                }
            }
            return R.success(categories);
        }


    }


    /**
     * 查询分类详情
     *
     * @param id 分类ID
     * @return 分类详情
     */
    @PostMapping("/findById")
    @ApiOperation("根据ID查询分类详情")
    public R findById(@ApiParam(name = "id", value = "分类ID", required = true) @RequestParam Integer id) {
        Category category = service.getById(id);
        return R.success(category);
    }

    /**
     * 批量删除分类
     *
     * @param ids 分类ID列表
     * @return 操作结果
     */
    @PostMapping("/batchDelete")
    @ApiOperation("批量删除分类")
    public R batchDeletion(@RequestBody List<Integer> ids) {
        if (ids == null || ids.isEmpty()) {
            return R.fail("请选择要删除的分类");
        }
        boolean delState = service.removeByIds(ids);
        return R.result(delState, delState ? "批量删除成功" : "批量删除失败");
    }


    // 常量定义
    private static final String SUCCESS_MSG = "未被引用";
    private static final String FAIL_MSG = "已被引用，确认删除将同时移除相关联的";

    /**
     * 校验：用于删除分类前校验文章是否引用过
     * 如果引用过就要提示“xxxx已被文章引用，是否确认删除，删除后对应文章内设置的分类将一并被删除！”，
     *
     * @param categoryId
     * @return
     */
    @PostMapping("/checkDelete")
    @ApiOperation("检查分类是否已被引用")
    public R checkDelete(@ApiParam("分类ID") @RequestParam Integer categoryId) {
        // 验证分类ID是否有效
        if (categoryId == null || categoryId <= 0) {
            return R.fail("无效的分类ID");
        }

        // 查询分类信息
        Category category = service.getById(categoryId);
        if (category == null) {
            return R.fail("分类不存在");
        }

        // 提取类型和站点ID
        Integer type = category.getType();
        Integer siteId = category.getSiteId();

        // 构建响应消息前缀
        String front = getFront(type);

        // 业务逻辑调用
        boolean isReferenced = service.checkDelete(siteId, categoryId, type);

        // 根据结果返回响应
        String message = isReferenced ? front + FAIL_MSG : front + SUCCESS_MSG + front;
        return isReferenced ? R.warn(message) : R.success(message);
    }

    // 辅助方法：根据类型构建响应消息前缀
    private String getFront(Integer type) {
        if (type == 0) {
            return "类型";
        } else if (type == 1) {
            return "标签/分类";
        }
        return ""; // 或者抛出异常，表明未知类型
    }


}
