package com.huoran.iasf.controller;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huoran.iasf.common.exception.BusinessException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.ProductDetails;
import com.huoran.iasf.entity.SysColumn;
import com.huoran.iasf.service.SysColumnService;
import com.huoran.iasf.service.SysProductDetailsService;
import com.huoran.iasf.service.SysSubjectSpecialtyService;
import com.huoran.iasf.vo.req.PageCourseProductReq;
import com.huoran.iasf.vo.resp.PageContentRespVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.Date;
import java.util.List;

/**
 * 综合产品与课程详情信息表控制类
 */
@RestController
@RequestMapping("/productDetails")
@Api(value = "综合产品与课程详情", tags = "综合产品与课程详情")
@Slf4j
public class SysProductDetailsController {

    @Autowired
    private SysProductDetailsService service;

    @Autowired
    private SysSubjectSpecialtyService subjectSpecialtyService;

    @Autowired
    private SysColumnService columnService;

    @PostMapping("/checkIfTheTitleIsRepeat")
    @ApiOperation(value = "标题判重——适用于新增、编辑及发布前的校验", response = Boolean.class)
    public R checkIfTheTitleIsRepeat(@RequestBody @Valid ProductDetails content) {
        boolean isDuplicate = false;

        // 如果ID不为空，说明是编辑操作，需要排除当前记录自身
        if (content.getId() != null) {
            isDuplicate = service.existsOtherByProductName(content.getName(), content.getId(), content.getSiteId());
        } else {
            // ID为空，说明是新增操作
            isDuplicate = service.existsByProductName(content.getName(), content.getSiteId());
        }

        return R.result(!isDuplicate, !isDuplicate ? "标题可用" : "该标题已重复!");
    }

    public static void main(String[] args) {
        // 获取当前日期时间
        LocalDate currentDate = LocalDate.now();

        // 输出当前日期，此时已经去除了时分秒信息
        System.out.println("当前日期（去除时分秒）: " + currentDate);
    }

    /**
     * 保存或更新综合产品与课程详情信息
     * 根据实体中的ID判断是保存还是更新
     *
     * @param productDetails 要保存或更新的实体
     * @return 操作结果
     */
    @PostMapping("/saveOrUpdate")
    @ApiOperation("保存或更新综合产品与课程详情信息")
    @Transactional(rollbackFor = Exception.class)
    public R saveOrUpdate(@RequestBody @Validated ProductDetails productDetails) {
        try {

            SysColumn column = columnService.getById(productDetails.getColumnId());
            if (column == null) {
                throw new BusinessException(BaseResponseCode.DATA_DOES_NOT_EXIST);
            }


            if (productDetails.getPublishStatus()) {
                // 获取当前日期时间
                Date currentDate = new Date();
                // 创建SimpleDateFormat对象，指定日期时间格式
                SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                // 格式化当前日期时间
                String formattedDateTime = formatter.format(currentDate);
                // 输出格式化后的日期时间
                productDetails.setReleaseTime(formattedDateTime);
            } else {
                productDetails.setReleaseTime("—");
            }
            boolean addState = service.saveOrUpdate(productDetails);


            if (addState && productDetails.getSubjectSpecialtyList() != null && !productDetails.getSubjectSpecialtyList().isEmpty()) {
                // 确保每个subjectSpecialty都关联到正确的templateDetailsId
                productDetails.getSubjectSpecialtyList().forEach(subjectSpecialty -> subjectSpecialty.setTemplateDetailsId(productDetails.getId()));
                subjectSpecialtyService.removeByTemplateDetailsId(productDetails.getId());
                subjectSpecialtyService.saveBatch(productDetails.getSubjectSpecialtyList());
            }

            //新增成功后返回id
            return R.result(addState, addState ? String.valueOf(productDetails.getId()) : "操作失败");
        } catch (Exception e) {
            log.error("保存或更新产品详情时发生错误", e);
            throw new BusinessException(BaseResponseCode.SYSTEM_BUSY);
        }
    }


    /**
     * 根据ID查询详情
     *
     * @param id 主键ID
     * @return 查询结果
     */
    @PostMapping("/findById")
    @ApiOperation("根据ID查询详情")
    public R findById(@RequestParam("id") Integer id) {
        ProductDetails productDetails = service.getById(id);

        //查询详情的时候也要查询学科专业
        productDetails.setSubjectSpecialtyList(subjectSpecialtyService.getByTemplateDetailsId(id));

        return R.success(productDetails);
    }


    //前台预览
    @PostMapping("/foregroundPreview")
    @ApiOperation("前台预览")
    public R foregroundPreview(@RequestParam("id") Integer id) {
        ProductDetails productDetails = service.foregroundPreview(id);
        return R.success(productDetails);
    }

   /* @PostMapping("/articlePreview")
    @ApiOperation(value = "增加文章浏览量(点击一次算一次)", response = ProductDetails.class)
    public R articlePreview(@ApiParam(name = "contentId", value = "文章id", required = true) @RequestParam Integer contentId) {
        return R.success(service.statisticsOfPageViews(contentId));
    }
*/

    @PostMapping("/articleEnableOrDisable")
    @ApiOperation(value = "文章启用禁用", response = ProductDetails.class)
    public R articleEnableOrDisable(@ApiParam(name = "id", value = "文章id", required = true) @RequestParam Integer id, @ApiParam(name = "isDisable", value = "是否禁用(0默认，0启用 1禁用)", required = true) @RequestParam Integer isDisable) {
        UpdateWrapper<ProductDetails> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("is_disable", isDisable);
        updateWrapper.eq("id", id);
        boolean ret = service.update(updateWrapper);
        return ret ? R.success() : R.fail("禁用/启用失败");
    }


    /**
     * 批量删除记录
     *
     * @param ids 主键ID列表
     * @return 操作结果
     */
    @PostMapping("/batchDelete")
    @ApiOperation("批量删除记录")
    public R batchDelete(@ApiParam(name = "ids", value = "主键", required = true) @RequestParam List<Integer> ids) {
        if (ids == null || ids.isEmpty()) {
            return R.fail("请选择要删除的记录");
        }
        boolean delState = service.removeByIds(ids);
        return R.result(delState, delState ? "批量删除成功" : "批量删除失败");
    }


    @PostMapping("/courseProductTopOperation")
    @ApiOperation(value = "课程产品置顶/取消置顶", response = ProductDetails.class)
    public R courseProductTopOperation(@ApiParam(name = "isTop", value = "是否置顶(默认为0 不置顶 1为置顶)", required = true) @RequestParam Integer isTop, @ApiParam(name = "articleId", value = "课程产品id", required = true) @RequestParam Integer articleId) {
        //是否置顶(默认为0 不置顶 1为置顶)
        UpdateWrapper<ProductDetails> updateWrap = new UpdateWrapper<>();

        if (isTop == 1) {
            //置顶时间(置顶一次更新一次)
            updateWrap.set("top_time", new Date());
        } else {
            updateWrap.set("top_time", null);
        }
        updateWrap.set("is_top", isTop);
        updateWrap.eq("id", articleId);
        updateWrap.eq("deleted", 1);
        boolean ret = service.update(updateWrap);
        return R.result(ret, ret ? "操作除成功" : "操作失败");

    }


    /**
     * 根据前端传的栏目id，只查询栏目下启用且未删除的数据
     *
     * @param sysContent
     * @return
     */
    @PostMapping("/courseProduct")
    @ApiOperation(value = "前台课程产品", response = PageContentRespVO.class)
    public R courseProduct(@RequestBody @Valid @ApiParam(name = "分页查询参数", value = "传入json格式", required = true) PageCourseProductReq sysContent) {
        return service.courseProduct(sysContent);
    }
}
