package com.huoran.iasf.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.*;
import com.huoran.iasf.mapper.SysColumnMapper;
import com.huoran.iasf.mapper.SysFooterSetupMapper;
import com.huoran.iasf.mapper.SysFooterSetupScopeOfApplicationMapper;
import com.huoran.iasf.service.SysFooterSetupService;
import com.huoran.iasf.vo.ConflictCheckResultVO;
import com.huoran.iasf.vo.FooterListVO;
import com.huoran.iasf.vo.SysPageFooterScopeOfApplicationWithSiteIdVO;
import com.huoran.iasf.vo.req.SuspensionBarListPagingReq;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 页脚配置 服务实现类
 * </p>
 *
 * @author cheney
 * @since 2024-06-07
 */
@Service
public class SysFooterSetupServiceImpl extends ServiceImpl<SysFooterSetupMapper, SysFooterSetup> implements SysFooterSetupService {
    @Resource
    private SysFooterSetupScopeOfApplicationMapper scopeOfApplicationMapper;

    @Autowired
    public SysColumnMapper columnMapper;

    @Override
    public R footerList(SuspensionBarListPagingReq sysFloatingColumnScheme) {
        Page<FooterListVO> page = new Page<FooterListVO>(sysFloatingColumnScheme.getPageNum(), sysFloatingColumnScheme.getPageSize());
        IPage<FooterListVO> pageList = baseMapper.footerList(page, sysFloatingColumnScheme);
        return R.success(pageList);
    }

    public R checkEnableOrDisable(Integer footerId, Integer enable) {
        SysFooterSetup scheme = baseMapper.selectById(footerId);
        if (scheme == null) {
            return R.fail("页脚方案不存在");
        }

        Integer siteId = scheme.getSiteId();

        // 1. 当前操作为启用时，检查逻辑
        if (enable == 0) {
            // 全局页脚启用逻辑
            // 对于启用操作，先检查是否是全局应用
            if (scheme.getIsGlobal() == 1) {
                // 检查站点下是否存在任何已启用的页脚方案，不论全局还是部分应用
                long enabledCount = baseMapper.selectCount(new LambdaQueryWrapper<SysFooterSetup>().eq(SysFooterSetup::getSiteId, siteId).eq(SysFooterSetup::getIsDisable, 0));
                if (enabledCount > 0) {
                    // 弹窗提示，表明启用新的全局页脚会覆盖当前已启用的页脚
                    return R.warn("当前站点已存在启用的页脚方案，启用新方案将会覆盖原有设置，请确认是否继续？");
                }
            } else { // 非全局页脚启用逻辑
                // 检查是否存在启用的全局页脚
                if (existsEnabledGlobal(siteId)) {
                    return R.fail("当前已启用全局页脚，无法直接启用部分应用的页脚，请先禁用全局页脚！");
                }
                // 检查与其它部分应用页脚的冲突
                if (hasOverlapWithOtherEnabled(footerId, siteId)) {
                    return R.fail(generateConflictMessage(footerId, siteId, enable));
                }
            }
        } else if (enable == 1) { // 禁用操作逻辑
            // 禁用操作直接允许，无需特殊检查
        } else {
            return R.fail("无效的操作类型");
        }

        // 如果没有冲突，直接返回成功提示
        return R.success("操作可执行");
    }

    @Override
    public R showTheFooterAccordingToTheColumn(String columnId) {
        // 获取栏目信息，提前返回错误信息如果栏目不存在
        SysColumn column = columnMapper.selectById(columnId);
        if (column == null) {
            return R.fail("未找到该栏目");
        }

        LambdaQueryWrapper<SysFooterSetup> queryWrapper = new LambdaQueryWrapper<SysFooterSetup>()
                .eq(SysFooterSetup::getSiteId, column.getSiteId())
                .eq(SysFooterSetup::getIsDisable, 0)
                .eq(SysFooterSetup::getStatus, 1);

        // 查询当前站点下已启用且已发布的页脚
        List<SysFooterSetup> floatingColumnSchemeList = baseMapper.selectList(queryWrapper);

        // 检查是否存在全局启用的页脚
        List<SysFooterSetup> globalSetupList = floatingColumnSchemeList.stream()
                .filter(setup -> setup.getIsGlobal() == 1)
                .collect(Collectors.toList());

        if (!globalSetupList.isEmpty()) {
            // 即使是全局方案，也包装成单元素的集合返回，以保持返回类型的统一
            return R.success(globalSetupList);
        }

        // 如果没有全局方案，检查当前栏目绑定的页脚
        List<Integer> boundSetupIds = scopeOfApplicationMapper.checkBoundByColumnId(columnId);
        if (!boundSetupIds.isEmpty()) {
            // 根据绑定的ID列表查询启用的页脚
            List<SysFooterSetup> setupsForColumn = baseMapper.selectList(
                    new LambdaQueryWrapper<SysFooterSetup>()
                            .in(SysFooterSetup::getId, boundSetupIds)
                            .eq(SysFooterSetup::getIsDisable, 0)
            );
            return R.success(setupsForColumn);
        }

        return R.success(Collections.emptyList());
    }



    @Transactional(readOnly = true)
    public ConflictCheckResultVO preCheckMultiScopeConflictBeforeUpdate(
            Integer siteId, List<SysFooterSetupScopeOfApplication> updatedScopes, Integer floatingBarSchemeId) {

        // 1. 收集待更新方案的ID，用于排除这些方案在冲突检测中的干扰
        List<String> excludeSchemeIds = updatedScopes.stream()
                .map(SysFooterSetupScopeOfApplication::getApplicationScopeId)
                .collect(Collectors.toList());

        // 2. 从数据库查询该站点下所有启用的页脚，但排除掉当前正在更新的那些方案
        List<SysPageFooterScopeOfApplicationWithSiteIdVO> enabledScopes =
                baseMapper.selectEnabledScopesForMultiConflictCheck(siteId, excludeSchemeIds,floatingBarSchemeId);

        // 初始化冲突标记为false，并准备一个StringBuilder用于记录冲突信息
        boolean hasConflict = false;
        StringBuilder conflictMessageBuilder = new StringBuilder();
        Set<String> conflictColumnNames = new HashSet<>(); // 用于存储所有冲突的栏目名称

        // 3. 对比每个待更新的方案与已启用方案，检测应用范围是否有重叠
        for (SysPageFooterScopeOfApplicationWithSiteIdVO existingScope : enabledScopes) {
            for (SysFooterSetupScopeOfApplication newScope : updatedScopes) {
                // 分割应用范围ID为集合，以便逐个比较
                Set<Integer> existingScopeIds = Arrays.stream(existingScope.getApplicationScopeId().split(","))
                        .map(String::trim)
                        .filter(id -> !id.isEmpty() && NumberUtils.isCreatable(id))
                        .map(Integer::parseInt)
                        .collect(Collectors.toSet());
                Set<Integer> newScopeIds = Arrays.stream(newScope.getApplicationScopeId().split(","))
                        .map(String::trim)
                        .filter(id -> !id.isEmpty() && NumberUtils.isCreatable(id))
                        .map(Integer::parseInt)
                        .collect(Collectors.toSet());

                // 找出重叠的范围ID
                Set<Integer> overlapIds = new HashSet<>(existingScopeIds);
                overlapIds.retainAll(newScopeIds);

                // 避免重复处理相同的冲突范围
                if (!overlapIds.isEmpty()) {
                    for (Integer overlapId : overlapIds) {
                        String columnName = getColumnNameById(String.valueOf(overlapId)); // 假设这是获取栏目名称的方法
                        conflictColumnNames.add(columnName); // 收集冲突的栏目名称
                    }
                }
            }
        }

        // 构建最终的冲突信息，如果存在冲突
        if (!conflictColumnNames.isEmpty()) {
            hasConflict = true;
            conflictMessageBuilder.append("栏目：");
            boolean isFirst = true;
            for (String columnName : conflictColumnNames) {
                if (!isFirst) {
                    conflictMessageBuilder.append("、"); // 使用中文逗号作为分隔符
                }
                conflictMessageBuilder.append(columnName);
                isFirst = false;
            }
            conflictMessageBuilder.append(" 已设置有页脚，请先移除相关设置再尝试启用。\n");
        }
        // 4. 根据检测结果创建并返回ConflictCheckResult对象
        return new ConflictCheckResultVO(hasConflict, conflictMessageBuilder.toString());
    }

    private String getColumnNameById(String overlapId) {
        return columnMapper.getColumnNamesById(overlapId);
    }


    /**
     * 检查当前页脚与已启用方案是否有重叠。
     *
     * @param id 页脚ID
     * @return 是否有重叠
     */
    private boolean hasOverlapWithOtherEnabled(Integer id, Integer siteId) {
        // 获取当前要检查的方案的应用范围
        List<SysFooterSetupScopeOfApplication> currentScopes = scopeOfApplicationMapper.selectList(new LambdaQueryWrapper<SysFooterSetupScopeOfApplication>().eq(SysFooterSetupScopeOfApplication::getFooterId, id));

        if (currentScopes.isEmpty()) {
            return false; // 当前方案无应用范围，自然无重叠
        }

        Set<Integer> currentScopeIds = currentScopes.stream().flatMap(scope -> Arrays.stream(scope.getApplicationScopeId().split(","))).map(Integer::parseInt).collect(Collectors.toSet());

        // 构造查询条件，确保比较的方案属于同一个站点
        String sqlInCondition = "SELECT id FROM sys_footer_setup WHERE is_disable = 0 AND site_id = " + siteId;

        // 查询所有启用且ID不等于当前id且属于相同站点的页脚的应用范围
        List<SysFooterSetupScopeOfApplication> enabledScopes = scopeOfApplicationMapper.selectList(new LambdaQueryWrapper<SysFooterSetupScopeOfApplication>().ne(SysFooterSetupScopeOfApplication::getFooterId, id).inSql(SysFooterSetupScopeOfApplication::getFooterId, sqlInCondition));

        // 检查是否有重叠
        for (SysFooterSetupScopeOfApplication enabledScope : enabledScopes) {
            Set<Integer> enabledScopeIds = Arrays.stream(enabledScope.getApplicationScopeId().split(",")).map(Integer::parseInt).collect(Collectors.toSet());

            if (!Collections.disjoint(currentScopeIds, enabledScopeIds)) {
                return true; // 存在重叠
            }
        }

        return false; // 无重叠
    }

    /**
     * 生成冲突提示信息，展示冲突的栏目名称。
     *
     * @param footerId 页脚ID
     * @return 冲突提示信息
     */
    private String generateConflictMessage(Integer footerId, Integer siteId, Integer enableAction) {
        // 确认当前页脚方案是否与其他方案存在范围重叠
        if (!hasOverlapWithOtherEnabled(footerId, siteId)) {
            return ""; // 若无重叠，直接返回空字符串表示无冲突
        }

        // 获取与当前页脚方案ID相关的冲突范围ID集合
        List<SysFooterSetupScopeOfApplication> conflictingScopes = scopeOfApplicationMapper.selectList(new LambdaQueryWrapper<SysFooterSetupScopeOfApplication>().eq(SysFooterSetupScopeOfApplication::getFooterId, footerId));

        // 将范围ID字符串转换为整数集合
        Set<Integer> conflictScopeIds = conflictingScopes.stream().flatMap(scope -> Arrays.stream(scope.getApplicationScopeId().split(","))).map(Integer::parseInt).collect(Collectors.toSet());

        //查询当前已经启用的页脚方案下的的应用范围跟当前设置的范围有重复的就只提出出来重复的id
        conflictScopeIds.retainAll(checkBoundByColumnId(conflictScopeIds, siteId));

        // 根据冲突范围ID查询对应的栏目名称
        List<String> conflictingScopeNames = columnMapper.selectList(new LambdaQueryWrapper<SysColumn>().in(SysColumn::getId, conflictScopeIds)).stream().map(SysColumn::getColumnName).collect(Collectors.toList());

        // 根据操作类型生成具体的提示信息
        if (enableAction == 0) { // 启用操作
            // 拼接冲突范围名称，提醒用户需先解决冲突
            return "栏目：“" + String.join("”， “", conflictingScopeNames) + "”已设置有页脚，请先移除相关设置再尝试启用。";
        } else {
            return "未知的操作类型";
        }
    }


    private Set<Integer> checkBoundByColumnId(Set<Integer> columnIds, Integer siteId) {
        // 构造SQL查询条件，用于查找所有启用的页脚方案及其范围
        /*String sqlInCondition = columnIds.stream().map(Object::toString).collect(Collectors.joining(","));*/ // 移除了外部的括号

        List<SysFooterSetupScopeOfApplication> enabledConflictingScopes = scopeOfApplicationMapper.selectList(new LambdaQueryWrapper<SysFooterSetupScopeOfApplication>()
                .inSql(SysFooterSetupScopeOfApplication::getFooterId,
                        "SELECT id FROM sys_footer_setup WHERE is_disable = 0 AND site_id = " + siteId).in(SysFooterSetupScopeOfApplication::getApplicationScopeId, columnIds));

        // 转换为范围ID集合
        Set<Integer> duplicateScopeIds = new HashSet<>();
        for (SysFooterSetupScopeOfApplication scope : enabledConflictingScopes) {
            duplicateScopeIds.addAll(Arrays.stream(scope.getApplicationScopeId().split(",")).map(Integer::parseInt).collect(Collectors.toSet()));
        }

        return duplicateScopeIds;
    }

    private boolean existsEnabledGlobal(Integer siteId) {

        QueryWrapper<SysFooterSetup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_global", 1);
        queryWrapper.eq("is_disable", 0);
        queryWrapper.eq("site_id", siteId);
        return !baseMapper.selectList(queryWrapper).isEmpty();

    }

}
